import { z } from 'zod';
import { Logger } from '../logger';
import { Telemetry } from '../telemetry';
import { Step } from './step';
import { WorkflowLogMessage, StepConfig, StepResult, RetryConfig } from './types';
export declare class Workflow<TSteps extends Step<any, any, any>[] = any, TTriggerSchema extends z.ZodType<any> = any> {
    #private;
    name: string;
    /**
     * Creates a new Workflow instance
     * @param name - Identifier for the workflow (not necessarily unique)
     * @param logger - Optional logger instance
     */
    constructor({ name, steps, logger, triggerSchema, retryConfig, telemetry, }: {
        name: string;
        logger?: Logger<WorkflowLogMessage>;
        steps: TSteps;
        triggerSchema?: TTriggerSchema;
        retryConfig?: RetryConfig;
        telemetry?: Telemetry;
    });
    /**
     * Initializes the XState machine for the workflow
     *
     * Registers the machine's types, actions, actors, initial context, entry actions, initial state, and states
     * @returns The initialized machine
     */
    private initializeMachine;
    /**
     * Configures a step in the workflow
     * @param id - Unique identifier for the step
     * @param config - Step configuration including handler, schema, variables, and payload
     * @returns this instance for method chaining
     */
    config<TStepId extends TSteps[number]['id']>(id: TStepId, config: StepConfig<TStepId, TSteps>): this;
    /**
     * Executes the workflow with the given trigger data
     * @param triggerData - Initial data to start the workflow with
     * @returns Promise resolving to workflow results or rejecting with error
     * @throws Error if trigger schema validation fails
     */
    execute(triggerData?: z.infer<TTriggerSchema>): Promise<{
        triggerData?: z.infer<TTriggerSchema>;
        results: Record<string, StepResult<any>>;
        runId: string;
    }>;
    /**
     * Rebuilds the machine with the current steps configuration and validates the workflow
     *
     * This is the last step of a workflow builder method chain
     * @throws Error if validation fails
     *
     * @returns this instance for method chaining
     */
    commit(): this;
}
